import numpy as np
from sklearn import tree
import pandas as pd
from sklearn.model_selection import train_test_split
import matplotlib.pyplot as plt
import datetime
import time

# Read time
def stringdate_to_datetime(stringdatetime: str):
 # transform a date string representation into an internal datetime representation
 # :param stringdatetime: date string representation '%d%m%Y %H:%M:%S'
 # :return: internal datetime representation
 if "." in stringdatetime:
  stringdatetime=stringdatetime.split(".")[0]
 else:
  stringdatetime=stringdatetime.split("Z")[0]
 return datetime.datetime.fromtimestamp(time.mktime(time.strptime(stringdatetime, '%Y-%m-%dT%H:%M:%S')))

# Read CSV file
data = pd.read_csv('outputChauffage.csv', sep=',')
data["datetime"]=data["time"].apply(stringdate_to_datetime)

data1 = pd.read_csv('outputCO2.csv', sep=',')
data1["datetime"]=data1["time"].apply(stringdate_to_datetime)

data2 = pd.read_csv('outputFenetre.csv', sep=',')
data2["datetime"]=data2["time"].apply(stringdate_to_datetime)

# # Hours - data = Occupancy
# hours=[]
# data_datetimes = pd.to_datetime(data.iloc[:,0], format='%Y-%m-%dT%H:%M:%S')
# for dt in data_datetimes:
#  hours.append(dt.hour)
# data['hour'] = hours
#
# occupancy = []
# for k in range(len(hours)):
#  if hours[k] < 8 or hours[k] > 21 : #8h à 21h occupancy = 1
#   occupancy.append(0)
#  else :
#   occupancy.append(1)
# data['occupancy'] = occupancy
# print(data)

# data
data.set_index('datetime', inplace=True)
print("isnull", data.isnull().any()) #find gaps
data.ffill()
Chauffage=data.resample('20T').sum()

# data1
data1.set_index('datetime', inplace=True)
print("isnull", data1.isnull().any()) #find gaps
data1.ffill()
CO2=data1.resample('20T').sum()

# data2
data2.set_index('datetime', inplace=True)
print("isnull", data2.isnull().any()) #find gaps
data2.ffill()
Fenetre=data2.resample('20T').sum()

# Merge data
mergeData=[Chauffage,CO2,Fenetre]
result=pd.concat(mergeData, axis=1, join='inner')
result.columns = ['chauffage', 'CO2', 'fenetre']

# Waste of energy
waste_of_energy = []
chauffage = result['chauffage']
CO2 = result['CO2']
fenetre = result['fenetre']
for k in range(len(chauffage)):
 if (chauffage[k]>0) and (CO2[k]<1000) and (fenetre[k]>0.2) :
  waste_of_energy.append(1)
 else :
  waste_of_energy.append(0)
result['waste_of_energy']= waste_of_energy

# Cleaning a little bit
index_with_nan = result.index[result.isnull().any(axis=1)]
result.drop(index_with_nan,0, inplace=True)

# Final result
result.to_csv (r'C:/Users/Hugo Phanthavong/PycharmProjects/ProjetG11/result.csv', index = False, header=True)

# from sklearn.cluster import KMeans
# def KMeans_clustering(jalousie):
#  myarray_jalousie = np.array(jalousie).reshape(len(jalousie),1)
#  km = KMeans(n_cluster=2, random_state=0)
#  km.fit(myarray_jalousie)
#  centers = km.cluster_centers_
#  centers[centers<0] = 0
#  prediction= km.fit_predict(myarray_jalousie)
#  return (prediction)

# outlier
# z_scores = stats.zscore(result)
# # calculate zscores of df
# abs_z_scores = np.abs(z_scores)
# filtered_entries = (abs_z_scores < 3).all(axis=1)
# new_df = result[filtered_entries]
# print(new_df)

###################################
########## Decision tree ##########
###################################

# Etude réalisée au préalable dans un autre cas que l'on a pas réussi à réadapter ici

import numpy as np
from sklearn import tree
import pandas as pd
from sklearn.model_selection import train_test_split
import matplotlib.pyplot as plt


# Read CSV file
data = pd.read_csv('/Users/Hugo Phanthavong/PycharmProjects/pythonProject1/data.csv', sep=';')

# Discretize the labels to (0,1)
newlabel = []
label = data['label']
for k in range(len(label)):
 if label[k] < 0.5 :
  newlabel.append(0)
 else :
  newlabel.append(1)
data['newlabel'] = newlabel

# Divide the data into training and testing (70/30 different days)
x_train, x_test, y_train, y_test = train_test_split(
 data[["Tin", "Tout", "humidity", "detected_motions", "power", "office_CO2_concentration", "Door", "CO2_corridor", "acoustic_pressure_dB"]],
 data[['newlabel']], test_size=.3, random_state=0)

# Define the most important features : feature_importances_
def importance(x_train, y_train):
 clf = tree.DecisionTreeClassifier(random_state=0, criterion='entropy', max_depth=None)
 clf = clf.fit(x_train, y_train)
 features_importance = clf.feature_importances_
 return features_importance

#Implementation of decision tree
def decision_tree(x_train, y_train, x_test, y_test):
 clf= tree.DecisionTreeClassifier(random_state=0, criterion='entropy', max_depth=None)
 clf= clf.fit(x_train, y_train)
 y_test_pred= clf.predict(x_test)
 features_importance = clf.feature_importances_
 print("features_importance", features_importance)
 #results : power, C02 office, door, CO2 corridor

 #Accuracy
 accuracy = clf.score(x_test,y_test)
 print("accuracy is", accuracy)

 #Plot the features importance
 indices = np.argsort(features_importance)
 plt.title('Feature_importances')
 plt.barh(range(len(indices)), features_importance[indices], color='b', align='center')
 plt.yticks(range(len(indices)), [x_train.columns.values[i] for i in indices])
 plt.xlabel('Relative Importance')
 plt.show()


 with open("tree.dot", "w") as file:
  tree.export_graphviz(clf, out_file=file)
 return(y_test_pred)

#Cost vs Accuracy from different scenarios
#cost=[Power, Motion, Micro]
c_price=[4*88.75, 119, 150]
#Accuracy_captors 1. Motion, 2. Motion & micro, 3. Motion & power, 4. all
cost_scenario=[c_price[1], c_price[1]+c_price[2], c_price[1]+c_price[0], c_price[0]+c_price[1]+c_price[2]]
accuracy_scenario=[0.948,0.957,0.970,0.978]

y_pred_occupancy = decision_tree(x_train, y_train, x_test, y_test)
